#test1.py
# A generic test..Using simple math to rotate and orbit planets
# ...Based loosely on the Solar-System sample
#   The idea behind the Solar System sampe is to show how linked nodes work,
#   This test is rather an excecise in game design concepts
# *..Create a generic game class template
# *..Encapsulate a setWindow method
# *..Encapsuulate generic methods to load/link models and textures
# *..Encapsulate a dictionary based reference to game ofjects
# *..Setup a simple Model class (setTag may provide more info.. investigating)
# *..Wrap the Model class methods in task calls
# Author: jestermonster@gmail.com
# 28Nov2009

import direct.directbase.DirectStart
from pandac.PandaModules import *
from direct.gui.DirectGui import *

import sys, math


#***********************************************************
class Model:
    #-------------------------------------------------------
    def __init__(self, model, position, scale):
        self.model = model
        self.position = position
        self.scale = scale
        self.rotation = (0,0,0)
        self.rotation_delta = (0,0,0)
        self.enable_rotation = False
        self.orbit_delta = 0
        self.enable_orbit = False
        self.orbit_distance = position[0] #assuming propper placement
        self.orbit_timer = 0
        self.orbit_center = ""

    #-------------------------------------------------------
    def setRotation(self, rotation):
        self.rotation = rotation
        self.model.setHpr(*rotation)

    #-------------------------------------------------------
    def setScale(self, scale):
        self.scale = scale
        self.model.setScale(scale)

    #-------------------------------------------------------
    def setPosition(self, position):
        self.position = position
        self.model.setPos(*position)

    #-------------------------------------------------------
    def setRotationDelta(self, delta):
        self.rotation_delta = delta
        self.enable_rotation = True

    #-------------------------------------------------------
    def rotateByDelta(self):
        if not self.enable_rotation:
            return
        x,y,z = self.rotation_delta
        xr,yr,zr = self.rotation
        xr+=x; yr+=y; zr+=z
        self.setRotation((xr,yr,zr))

    #-------------------------------------------------------
    def setOrbitDelta(self, delta):
        self.orbit_delta = delta
        self.enable_orbit = True

    #-------------------------------------------------------
    def orbitByDelta(self):
        if not self.enable_orbit:
            return
        if len(self.orbit_center): #dont orbit moons around 0,0,0
            return
        radius = self.orbit_distance
        self.orbit_timer += self.orbit_delta
        x = radius*math.cos(self.orbit_timer)
        y = radius*math.sin(self.orbit_timer)
        z=0
        self.setPosition((x,y,z))

    #-------------------------------------------------------
    def setOrbitDistance(self, distance):
        self.orbit_distance = distance

    #-------------------------------------------------------
    def setOrbitCenter(self, center):
        self.orbit_center = center

    #-------------------------------------------------------
    def orbitMoon(self,center):
        xc,yc,zc = center
        radius = self.orbit_distance
        self.orbit_timer += self.orbit_delta
        x = radius*math.cos(self.orbit_timer)
        y = radius*math.sin(self.orbit_timer)
        z=0
        #offset results to orbit center
        x+=xc; y+=yc; z+=zc
        self.setPosition((x,y,z))


#***********************************************************
class World:
    #-------------------------------------------------------
    def __init__(self):
        self.models = {}
        self.textures = {}
        self.setCameraLookat((0,-30,20),(0,0,0))
        self.setWindow("Test 1", (0,0,0))
        self.loadModels()
        self.setupAnimations()

    #-------------------------------------------------------
    def setWindow(self, title, bgcolor):
        base.disableMouse() #detatch camera from mouse control
        base.setBackgroundColor(*bgcolor)
        winProps = WindowProperties()
        winProps.setTitle(title)
        base.win.requestProperties(winProps)

    #-------------------------------------------------------
    def setCamera(self, position, rotation):
        self.setCameraPosition(position)
        self.setCameraRotation(rotation)

    #-------------------------------------------------------
    def setCameraLookat(self, position, target):
        self.setCameraPosition(position)
        self.setCameraTarget(target)

    #-------------------------------------------------------
    def setCameraPosition(self, position):
        camera.setPos (*position)

    #-------------------------------------------------------
    def setCameraRotation(self, rotation):
        camera.setHpr (*rotation)

    #-------------------------------------------------------
    def setCameraTarget(self, target):
        base.camera.lookAt(*target)

    #-------------------------------------------------------
    def loadTexture(self, name, texturefile):
        texture = loader.loadTexture(texturefile)
        self.textures[name] = texture

    #-------------------------------------------------------
    def setTexture(self, modelname, texturename):
        self.models[modelname].model.setTexture(self.textures[texturename],1)

    #-------------------------------------------------------
    def loadModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.loadModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render)
        model.setScale(scale)
        model.setPos(*position)
        self.models[name] = Model(model, position, scale)

    #-------------------------------------------------------
    def loadModels(self):
        sizescale = 0.6
        orbitscale = 10
        self.loadModelAndTexture("sky","models/solar_sky_sphere","models/stars_1k_tex.jpg",(0,0,0),40)
        self.loadModelAndTexture("sun","models/planet_sphere","models/sun_1k_tex.jpg",(0,0,0),2*sizescale)
        self.loadModelAndTexture("venus","models/planet_sphere","models/venus_1k_tex.jpg",(0.72 * orbitscale, 0, 0), 0.923 * sizescale)
        self.loadModelAndTexture("mars","models/planet_sphere","models/mars_1k_tex.jpg",(1.52 * orbitscale, 0, 0), 0.515 * sizescale)
        self.loadModelAndTexture("earth","models/planet_sphere","models/earth_1k_tex.jpg",(orbitscale, 0, 0),(sizescale))
        self.loadModelAndTexture("moon","models/planet_sphere","models/moon_1k_tex.jpg",(1.1 *orbitscale, 0, 0), 0.3 *sizescale)

    #-------------------------------------------------------
    def setupAnimations(self):
        self.models["sun"].setRotationDelta((0.1, 0, 0))
        self.models["venus"].setRotationDelta((0.9, 0, 0))
        self.models["mars"].setRotationDelta((0.5, 0, 0))
        self.models["earth"].setRotationDelta((0.7, 0, 0))
        self.models["moon"].setRotationDelta((1, 0, 0))
        taskMgr.add(self.task_rotatePlanets, "rotate planets")
        self.models["venus"].setOrbitDelta(0.0015)
        self.models["mars"].setOrbitDelta(0.0006)
        self.models["earth"].setOrbitDelta(0.0009)
        self.models["moon"].setOrbitDelta(0.01)
        self.models["moon"].setOrbitCenter("earth") #planet to orbit
        self.models["moon"].setOrbitDistance(1) #distance from earth
        taskMgr.add(self.task_orbitPlanets,"orbit planets")

    #-------------------------------------------------------
    def task_rotatePlanets(self, task):
        for p in self.models:
            self.models[p].rotateByDelta()
        return task.cont

    #-------------------------------------------------------
    def task_orbitPlanets(self, task):
        for p in self.models:
            if len(self.models[p].orbit_center):
                planet = self.models[p].orbit_center
                self.models[p].orbitMoon(self.models[planet].position)
            else:
                self.models[p].orbitByDelta()
        return task.cont

    #-------------------------------------------------------



#***********************************************************
if __name__ == '__main__':
    w = World()
    run()