#test2.py
# A generic test..Playing around with spacial motion
# *..Using a dictionary to hold model references (Not using Model class from test1)
# *..Using setTag to hold model specific Metadata
# *..Added method getTagFloat to return a float value from a tag
# *..Testing timed tasks with doMethodLater
# *..Testing intervals to move a model
# *..Using LineNodePath to trace model motion
# *..Using lookAt to rotate models
# *..Using quaternions to steer model (code obtained from Panda3D forum)
# *..Added 'direct tools' usage option
# *..Added simple FULLSCREEN option
# Author: jestermonster@gmail.com
# 28Nov2009

from pandac.PandaModules import loadPrcFileData

#using direct tools ... remove before distribution
DTOOLS = False
if DTOOLS:
    loadPrcFileData("", "want-tk #t")
#end direct tools

#Fullsceen mode option
FULLSCREEN = False
if FULLSCREEN:
    loadPrcFileData("", "fullscreen #t")
else:
    loadPrcFileData("", "fullscreen #f")

import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.interval.LerpInterval import *
from pandac.PandaModules import *
from direct.gui.DirectGui import *
from direct.directtools.DirectGeometry import LineNodePath

import sys, math, random



#***********************************************************
class World(DirectObject):
    #-------------------------------------------------------
    def __init__(self):
        self.models = {}
        self.textures = {}
        self.changeQuat = Quat()
        self.setCameraLookat((0,-50,10),(0,0,0)) #back 50, yp 10
        self.setWindow("Test 2", (0,0,0))
        self.loadModels()
        self.setupAnimations()

    #-------------------------------------------------------
    def setWindow(self, title, bgcolor):
        base.disableMouse() #detatch camera from mouse control
        base.setBackgroundColor(*bgcolor)
        winProps = WindowProperties()
        winProps.setTitle(title)
        base.win.requestProperties(winProps)

    #-------------------------------------------------------
    def setCamera(self, position, rotation):
        self.setCameraPosition(position)
        self.setCameraRotation(rotation)

    #-------------------------------------------------------
    def setCameraLookat(self, position, target):
        self.setCameraPosition(position)
        self.setCameraTarget(target)

    #-------------------------------------------------------
    def setCameraPosition(self, position):
        camera.setPos (*position)

    #-------------------------------------------------------
    def setCameraRotation(self, rotation):
        camera.setHpr (*rotation)

    #-------------------------------------------------------
    def setCameraTarget(self, target):
        base.camera.lookAt(*target)

    #-------------------------------------------------------
    def setCameraToModel(self,model):
        camera.reparentTo(self.models[model])

    #-------------------------------------------------------
    def loadTexture(self, name, texturefile):
        texture = loader.loadTexture(texturefile)
        self.textures[name] = texture

    #-------------------------------------------------------
    def setTexture(self, modelname, texturename):
        self.models[modelname].setTexture(self.textures[texturename],1)

    #-------------------------------------------------------
    def loadModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.loadModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render)
        model.setScale(scale)
        model.setPos(*position)
        model.setTag("name",name) #using tag to store Meta info
        self.models[name] = model

    #-------------------------------------------------------
    def loadModels(self):
        sizescale = 0.6
        orbitscale = 10
        self.loadModelAndTexture("sky","models/solar_sky_sphere","models/stars_1k_tex.jpg",(0,0,0),40)
        self.loadModelAndTexture("sun","models/planet_sphere","models/sun_1k_tex.jpg",(0,0,0),1.5*sizescale)
        self.loadModelAndTexture("earth","models/planet_sphere","models/earth_1k_tex.jpg",(orbitscale, 0, 0),(sizescale))
        self.loadModelAndTexture("ball","models/planet_sphere","models/moon_1k_tex.jpg",(13,0,0),2*sizescale)
        self.loadModelAndTexture("pointer","models/pointer","models/wood1.jpg",(0,0,0),1)
        self.loadModelAndTexture("pointer2","models/pointer","models/moon_1k_tex.jpg",(0,0,0),1)
        self.loadModelAndTexture("pointer3","models/eagleship","models/eagletxtr.jpg",(0,-25,0),0.2)

    #-------------------------------------------------------
    def setTag(self,model,tag,value):
        self.models[model].setTag(tag,str(value))

    #-------------------------------------------------------
    def getTag(self,model,tag):
        return self.models[model].getTag(tag)

    #-------------------------------------------------------
    def getTagFloat(self,model,tag):
        return float(self.models[model].getTag(tag))

    #-------------------------------------------------------
    def setupAnimations(self):
        self.setTag("earth","orbit_distance",self.models["earth"].getX())
        self.setTag("earth","timer",0)
        self.setTag("earth","orbit_delta",0.005)
        self.setTag("pointer3","speed",0.02) #speed of pointer-spaceship
        self.line = LineNodePath(parent = render, thickness = 3.0, colorVec = Vec4(1, 0, 0, 1))
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        #To use 3rd person camera view of 'pointer3', uncomment next line
        #self.setCameraToModel("pointer3")
        taskMgr.add(self.task_doAnimations,"do aimations")
        taskMgr.doMethodLater(10, self.timedtask_moveRandom, "random ball")
        taskMgr.add(self.task_quadSteering,"quads")

    #-------------------------------------------------------
    def task_doAnimations(self,task):
        radius = self.getTagFloat("earth","orbit_distance")
        timer = self.getTagFloat("earth","timer")
        delta = self.getTagFloat("earth","orbit_delta")
        timer += delta
        x = radius*math.cos(timer)
        y = radius*math.sin(timer)
        z=0
        self.setTag("earth","timer",timer)
        self.models["earth"].setPos((x,y,z))
        self.models["pointer2"].lookAt(self.models["earth"].getPos())
        self.models["pointer"].lookAt(self.models["ball"].getPos())
        nx = self.models["ball"].getX()
        ny = self.models["ball"].getY()
        nz = self.models["ball"].getZ()
        #to make camera follw "ball", uncomment the next line
        #self.setCameraTarget((nx,ny,nz))
        self.line.reset()
        self.line.drawLines([((self.ox, self.oy, self.oz),(nx,ny,nz))])
        self.line.create()
        return task.cont

    #-------------------------------------------------------
    def timedtask_moveRandom(self, task):
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        x = random.randint(-12,12)
        y = random.randint(-12,12)
        z = random.randint(-12,12)
        ballmotion = self.models["ball"].posInterval(3,Point3(x,y,z))
        ballmotion.start()
        return task.again

    #-------------------------------------------------------
    def task_quadSteering(self, task):
        """Move mouse around.. using center of screen as pivot point..
        this steers and flies 'pointer3' around using quaternion calculations"""
        SQUARERADIUS = .7
        if base.mouseWatcherNode.hasMouse():
            x = base.mouseWatcherNode.getMouseX()
            y = base.mouseWatcherNode.getMouseY()
        else:
            x = 0; y = 0
        percentX = x / SQUARERADIUS
        percentY = y / SQUARERADIUS
        if percentX > 1:
            percentX = 1
        if percentY > 1:
            percentY = 1
        if percentX < -1:
            percentX = -1
        if percentY < -1:
            percentY = -1
        self.newP = percentY
        self.newH = -percentX
        self.newR = 0
        self.speed = self.getTagFloat("pointer3","speed")
        self.changeQuat.setHpr(VBase3(self.newH, self.newP, self.newR))
        model = self.models["pointer3"]
        model.setQuat(self.changeQuat * model.getQuat())
        self.velocityVector = model.getQuat().getForward() * self.speed
        model.setPos(  self.velocityVector[0] + model.getX(),
                            self.velocityVector[1] + model.getY(),
                            self.velocityVector[2] + model.getZ())
        return task.cont



#***********************************************************
if __name__ == '__main__':
    w = World()
    run()
