# test4.py
#
# Testing Space flight and render2D
# *..Added Debug to dump trace data onscreen
# *..Added keyboard controls
# *..Added camera toggle to toglle to ship cockpit and solar view
# *..Added load2dModel method to attach model to render2d
# *..Changed model scales and motion speeds
# Keep mouse in sights to fly straight
# Move mouse around to turn ship (up/ down/ left/ right : relatove to sights)
# Left and Right keys rotate ship
# Up ans down keys control ship speed
# q key toggles cockpit and solar view
# ESC key terminates program
#
# Author: jestermonster@gmail.com
#         jestermon.weebly.com
# 30Nov2009

from pandac.PandaModules import loadPrcFileData

#using direct tools ... remove before distribution
DTOOLS = False
if DTOOLS:
    loadPrcFileData("", "want-tk #t")
#end direct tools

#Fullsceen mode option
FULLSCREEN = False
if FULLSCREEN:
    loadPrcFileData("", "fullscreen #t")
else:
    loadPrcFileData("", "fullscreen #f")

import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.interval.LerpInterval import *
from pandac.PandaModules import *
from direct.gui.DirectGui import *
from direct.directtools.DirectGeometry import LineNodePath
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import TextNode

import sys, math, random





#***********************************************************
class World(DirectObject):
    #-------------------------------------------------------
    def __init__(self):
        self.models = {}
        self.textures = {}
        self.keys = {}
        self.changeQuat = Quat()
        self.setupDefaults()
        self.setCameraLookat((0,-40,10),(0,0,0)) #back 50, yp 10
        self.setWindow("Test 4", (0,0,0))
        self.loadModels()
        self.setupKeys()
        self.setupAnimations()

    #-------------------------------------------------------
    def debug(self,text):
        txt = str(text)
        self.debugtext.setText(txt)

    #-------------------------------------------------------
    def setupDefaults(self):
        self.BLACK = Vec4(0,0,0,1)
        self.WHITE = Vec4(1,1,1,1)

        self.font = loader.loadFont("models/cmss12")
        self.debugtext = OnscreenText(text="Debug: ",
                              style=1, fg=(1,1,1,1), font = self.font,
                              pos=(-1.0, 0.95), scale = .07)


    #-------------------------------------------------------
    def setWindow(self, title, bgcolor):
        base.disableMouse() #detatch camera from mouse control
        base.setBackgroundColor(*bgcolor)
        winProps = WindowProperties()
        winProps.setTitle(title)
        base.win.requestProperties(winProps)

    #-------------------------------------------------------
    def setupKeys(self):
        self.ctoggle = 0
        self.accept("escape", sys.exit)
        self.keys ={"Left":0, "Right":0, "Up":0, "Down":0, "PgUp":0, "PgDn":0}
        self.accept("arrow_left", self.setKey,["Left",1])     #left-pressed
        self.accept("arrow_left-up", self.setKey,["Left",0])  #left-released
        self.accept("arrow_right", self.setKey,["Right", 1])
        self.accept("arrow_right-up", self.setKey,["Right",0])
        self.accept("arrow_up", self.setKey, ["Up", 1])
        self.accept("arrow_up-up", self.setKey, ["Up", 0])
        self.accept("arrow_down", self.setKey, ["Down",1])
        self.accept("arrow_down-up", self.setKey, ["Down",0])
        self.accept("page_up", self.setKey, ["PgUp",1])
        self.accept("page_up-up", self.setKey, ["PgUp",0])
        self.accept("page_down", self.setKey, ["PgDn",1])
        self.accept("page_down-up", self.setKey, ["PgDn",0])

        self.accept("q", self.toggleCamera)


    #-------------------------------------------------------
    def toggleCamera(self):
        if self.ctoggle:
            self.ctoggle = 0
            camera.reparentTo(render)
            self.setCameraLookat((0,-40,10),(0,0,0))
            return
        else:
            #attatch camera to ship
            self.ctoggle = 1
            self.setCameraToModel("ship")
            self.setCameraLookat((0,-30, 12), (0,1000,0))
            return

    #-------------------------------------------------------
    def setKey(self, key, val):
        self.keys[key] = val

    #-------------------------------------------------------
    def setCamera(self, position, rotation):
        self.setCameraPosition(position)
        self.setCameraRotation(rotation)

    #-------------------------------------------------------
    def setCameraLookat(self, position, target):
        self.setCameraPosition(position)
        self.setCameraTarget(target)

    #-------------------------------------------------------
    def setCameraPosition(self, position):
        camera.setPos (*position)

    #-------------------------------------------------------
    def setCameraRotation(self, rotation):
        camera.setHpr (*rotation)

    #-------------------------------------------------------
    def setCameraTarget(self, target):
        base.camera.lookAt(*target)

    #-------------------------------------------------------
    def setCameraToModel(self,model):
        camera.reparentTo(self.models[model])

    #-------------------------------------------------------
    def loadTexture(self, name, texturefile):
        texture = loader.loadTexture(texturefile)
        self.textures[name] = texture

    #-------------------------------------------------------
    def setTexture(self, modelname, texturename):
        self.models[modelname].setTexture(self.textures[texturename],1)

    #-------------------------------------------------------
    def loadModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.loadModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render)
        model.setScale(scale)
        model.setPos(*position)
        model.setTag("name",name) #using tag to store Meta info
        self.models[name] = model

    #-------------------------------------------------------
    def load2dModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render2d)
        model.setScale(scale)
        model.setPos(*position)
        model.setTag("name",name) #using tag to store Meta info
        self.models[name] = model

    #-------------------------------------------------------
    def load2dModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.load2dModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModels(self):
        sizescale = 0.6
        orbitscale = 10
        self.loadModelAndTexture("sky","models/solar_sky_sphere","models/stars_1k_tex.jpg",(0,0,0),200)
        self.loadModelAndTexture("sun","models/planet_sphere","models/sun_1k_tex.jpg",(0,0,0),3)
        self.loadModelAndTexture("earth","models/planet_sphere","models/earth_1k_tex.jpg",(orbitscale, 0, 0),0.1)
        self.loadModelAndTexture("ball","models/planet_sphere","models/moon_1k_tex.jpg",(13,0,0),0.2)
        self.loadModelAndTexture("ship","models/cockpit1","models/ship-tx2.jpg",(0,-20,0),0.0005)
        self.load2dModel("sights","models/sights",(0,0,0),0.005)


    #-------------------------------------------------------
    def setTag(self,model,tag,value):
        self.models[model].setTag(tag,str(value))

    #-------------------------------------------------------
    def getTag(self,model,tag):
        return self.models[model].getTag(tag)

    #-------------------------------------------------------
    def getTagFloat(self,model,tag):
        return float(self.models[model].getTag(tag))

    #-------------------------------------------------------
    def setupAnimations(self):
        self.setTag("earth","orbit_distance",self.models["earth"].getX())
        self.setTag("earth","timer",0)
        self.setTag("earth","orbit_delta",0.0005)
        self.setTag("ship","speed",0.0) #speed of ship
        self.line = LineNodePath(parent = render, thickness = 1.0, colorVec = Vec4(1, 0, 0, 1))
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        taskMgr.add(self.task_doAnimations,"do aimations")
        taskMgr.doMethodLater(10, self.timedtask_moveRandom, "random ball")
        taskMgr.add(self.task_quadSteering1,"quads")
        taskMgr.add(self.task_kbSteering, "kbsteering")

    #-------------------------------------------------------
    def task_doAnimations(self,task):
        radius = self.getTagFloat("earth","orbit_distance")
        timer = self.getTagFloat("earth","timer")
        delta = self.getTagFloat("earth","orbit_delta")
        timer += delta
        x = radius*math.cos(timer)
        y = radius*math.sin(timer)
        z=0
        self.setTag("earth","timer",timer)
        self.models["earth"].setPos((x,y,z))
        nx = self.models["ball"].getX()
        ny = self.models["ball"].getY()
        nz = self.models["ball"].getZ()
        self.line.reset()
        self.line.drawLines([((self.ox, self.oy, self.oz),(nx,ny,nz))])
        self.line.create()
        return task.cont

    #-------------------------------------------------------
    def timedtask_moveRandom(self, task):
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        x = random.randint(-12,12)
        y = random.randint(-12,12)
        z = random.randint(-12,12)
        ballmotion = self.models["ball"].posInterval(3,Point3(x,y,z))
        ballmotion.start()
        return task.again

    #-------------------------------------------------------
    def changeShipSpeed(self, delta):
        speed = self.getTagFloat("ship","speed")
        speed += delta
        if speed > 0.05:
            speed = 0.05
        if speed < 0:
            speed = 0
        self.setTag("ship","speed",speed)

    #-------------------------------------------------------
    def task_quadSteering1(self, task):
        """Move mouse around.. using center of screen as pivot point..
        this steers and flies 'pointer3' around using quaternion calculations"""
        SQUARERADIUS = 0.7
        percentZ = 0
        moveval = 0.2
        speed = self.getTagFloat("ship","speed")
        self.debug("Speed: " + str(speed))
        if base.mouseWatcherNode.hasMouse():
            x = base.mouseWatcherNode.getMouseX()
            y = base.mouseWatcherNode.getMouseY()
        else:
            x = 0; y = 0
        percentX = x / SQUARERADIUS * moveval
        percentY = y / SQUARERADIUS * moveval
        if percentX > moveval:
            percentX = moveval
        if percentY > moveval:
            percentY = moveval
        if percentX < -moveval:
            percentX = -moveval
        if percentY < -moveval:
            percentY = -moveval
        self.newP = percentY
        self.newH = -percentX
        self.newR = percentZ

        self.changeQuat.setHpr(VBase3(self.newH, self.newP, self.newR))
        model = self.models["ship"]
        model.setQuat(self.changeQuat * model.getQuat())
        self.velocityVector = model.getQuat().getForward() * speed
        model.setPos(  self.velocityVector[0] + model.getX(),
                            self.velocityVector[1] + model.getY(),
                            self.velocityVector[2] + model.getZ())
        return task.cont

    #-------------------------------------------------------
    def task_kbSteering(self,task):
        if self.keys["Left"]:
            rot = self.models["ship"].getR()
            self.models["ship"].setR(rot - 0.2)
        if self.keys["Right"]:
            rot = self.models["ship"].getR()
            self.models["ship"].setR(rot + 0.2)
        if self.keys["Up"]:
            self.changeShipSpeed(0.00005)
        if self.keys["Down"]:
            self.changeShipSpeed(-0.00005)
        if self.keys["PgUp"]:
            pass
        if self.keys["PgDn"]:
            pass
        return task.cont



#***********************************************************
if __name__ == '__main__':
    w = World()
    run()
