# test4.py
#
# Testing Auto-guidance for a spaceship
# *..Use Hidden pointer to lookat the target
# *..Use interval to rotate ship to Hpr of the pointer
# *..toggle view with 'q'
# Author: jestermonster@gmail.com
#         jestermon.weebly.com
# 30Nov2009

from pandac.PandaModules import loadPrcFileData

#using direct tools ... remove before distribution
DTOOLS = False
if DTOOLS:
    loadPrcFileData("", "want-tk #t")
#end direct tools

#Fullsceen mode option
FULLSCREEN = False
if FULLSCREEN:
    loadPrcFileData("", "fullscreen #t")
else:
    loadPrcFileData("", "fullscreen #f")

import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.interval.LerpInterval import *
from pandac.PandaModules import *
from direct.gui.DirectGui import *
from direct.directtools.DirectGeometry import LineNodePath
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import TextNode

import sys, math, random





#***********************************************************
class World(DirectObject):
    #-------------------------------------------------------
    def __init__(self):
        self.models = {}
        self.textures = {}
        self.keys = {}
        self.changeQuat = Quat()
        self.setupDefaults()
        self.setCameraLookat((0,-50,20),(0,0,0))
        self.setWindow("Test 5", (0,0,0))
        self.loadModels()
        self.setupKeys()
        self.setupAnimations()

    #-------------------------------------------------------
    def debug(self,text):
        txt = str(text)
        self.debugtext.setText(txt)

    #-------------------------------------------------------
    def setupDefaults(self):
        self.BLACK = Vec4(0,0,0,1)
        self.WHITE = Vec4(1,1,1,1)

        self.font = loader.loadFont("models/cmss12")
        self.debugtext = OnscreenText(text="Debug: ",
                              style=1, fg=(1,1,1,1), font = self.font,
                              pos=(-0.60, 0.95), scale = .07)


    #-------------------------------------------------------
    def setWindow(self, title, bgcolor):
        base.disableMouse() #detatch camera from mouse control
        base.setBackgroundColor(*bgcolor)
        winProps = WindowProperties()
        winProps.setTitle(title)
        base.win.requestProperties(winProps)

    #-------------------------------------------------------
    def setupKeys(self):
        self.ctoggle = 0
        self.accept("escape", sys.exit)
        self.accept("q", self.toggleCamera)


    #-------------------------------------------------------
    def toggleCamera(self):
        if self.ctoggle:
            self.ctoggle = 0
            camera.reparentTo(render)
            self.setCameraLookat((0,-50,20),(0,0,0))
            return
        else:
            #attatch camera to ship
            self.ctoggle = 1
            self.setCameraToModel("ship")
            self.setCameraLookat((0,-40, 10), (0,0,0))
            return

    #-------------------------------------------------------
    def setKey(self, key, val):
        self.keys[key] = val

    #-------------------------------------------------------
    def setCamera(self, position, rotation):
        self.setCameraPosition(position)
        self.setCameraRotation(rotation)

    #-------------------------------------------------------
    def setCameraLookat(self, position, target):
        self.setCameraPosition(position)
        self.setCameraTarget(target)

    #-------------------------------------------------------
    def setCameraPosition(self, position):
        camera.setPos (*position)

    #-------------------------------------------------------
    def setCameraRotation(self, rotation):
        camera.setHpr (*rotation)

    #-------------------------------------------------------
    def setCameraTarget(self, target):
        base.camera.lookAt(*target)

    #-------------------------------------------------------
    def setCameraToModel(self,model):
        camera.reparentTo(self.models[model])

    #-------------------------------------------------------
    def loadTexture(self, name, texturefile):
        texture = loader.loadTexture(texturefile)
        self.textures[name] = texture

    #-------------------------------------------------------
    def setTexture(self, modelname, texturename):
        self.models[modelname].setTexture(self.textures[texturename],1)

    #-------------------------------------------------------
    def loadModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.loadModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render)
        model.setScale(scale)
        model.setPos(*position)
        model.setTag("name",name) #using tag to store Meta info
        self.models[name] = model

    #-------------------------------------------------------
    def load2dModel(self, name, modelfile, position, scale):
        model = loader.loadModel(modelfile)
        model.reparentTo(render2d)
        model.setScale(scale)
        model.setPos(*position)
        model.setTag("name",name) #using tag to store Meta info
        self.models[name] = model

    #-------------------------------------------------------
    def load2dModelAndTexture(self,name,modelfile,texturefile,position,scale):
        self.load2dModel(name,modelfile,position,scale)
        self.loadTexture(name,texturefile)
        self.setTexture(name,name)

    #-------------------------------------------------------
    def loadModels(self):
        sizescale = 0.6
        orbitscale = 10
        self.loadModelAndTexture("sky","models/solar_sky_sphere","models/stars_1k_tex.jpg",(0,0,0),200)
        self.loadModelAndTexture("ball","models/planet_sphere","models/moon_1k_tex.jpg",(self.randomPos()),0.5)
        self.loadModelAndTexture("ship","models/eagleship","models/eagletxtr.jpg",(0,-10,0),0.2)
        self.loadModelAndTexture("pointer","models/pointer","models/venus_1k_tex.jpg",(0,0,0),0.5)

    #-------------------------------------------------------
    def setTag(self,model,tag,value):
        self.models[model].setTag(tag,str(value))

    #-------------------------------------------------------
    def getTag(self,model,tag):
        return self.models[model].getTag(tag)

    #-------------------------------------------------------
    def getTagFloat(self,model,tag):
        return float(self.models[model].getTag(tag))

    #-------------------------------------------------------
    def setupAnimations(self):
        self.setTag("ship","speed",0.02) #speed of ship
        self.line = LineNodePath(parent = render, thickness = 1.0, colorVec = Vec4(1, 0, 0, 1))
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        taskMgr.add(self.task_doAnimations,"do aimations")
        taskMgr.doMethodLater(7, self.timedtask_moveRandom, "random ball")
        #comment out next line to see pointer
        self.models["pointer"].hide()
        taskMgr.add(self.task_autoGuidance,"auto guidance")

    #-------------------------------------------------------
    def task_doAnimations(self,task):
        nx = self.models["ball"].getX()
        ny = self.models["ball"].getY()
        nz = self.models["ball"].getZ()
        self.line.reset()
        self.line.drawLines([((self.ox, self.oy, self.oz),(nx,ny,nz))])
        self.line.create()
        return task.cont

    #-------------------------------------------------------
    def randomPos(self):
        x = random.randint(-11,11)
        y = random.randint(-11,11)
        z = random.randint(-11,11)
        return (x,y,z)

    #-------------------------------------------------------
    def timedtask_moveRandom(self, task):
        self.ox = self.models["ball"].getX()
        self.oy = self.models["ball"].getY()
        self.oz = self.models["ball"].getZ()
        x,y,z = self.randomPos()
        ballmotion = self.models["ball"].posInterval(3,Point3(x,y,z))
        ballmotion.start()
        return task.again

    #-------------------------------------------------------
    def task_autoGuidance(self,task):
        self.homingFlight("ship","ball")
        return task.cont

    #-------------------------------------------------------
    def homingFlight(self,source_model,target_model):
        source = self.models[source_model]
        model = source
        target = self.models[target_model]
        pointer = self.models["pointer"]
        pointer.setPos(source.getPos())
        pointer.lookAt(target.getPos())
        sx,sy,sz = source.getPos()
        tx,ty,tz = target.getPos()-source.getPos()
        if tz-sz < 0: #gets rid of a bit of flutter
            sz+=0.001
            source.setPos(sx,sy,sz)
        if tz-sz >0: #gets rid of a bit of flutter
            sz-=0.001
            source.setPos(sx,sy,sz)
        speed = self.getTagFloat(source_model,"speed")
        dx,dy,dz = source.getPos() - target.getPos()
        stop_distance = 0.5
        if abs(dx) < stop_distance and abs(dy) < stop_distance and abs(dz) < stop_distance:
            if self.homing_rotation.isPlaying():
                self.homing_rotation.finish()
            return
        h,p,r = pointer.getHpr()
        self.homing_rotation = source.hprInterval(2,(h,p,0))
        self.homing_rotation.start()
        self.changeQuat.setHpr(VBase3(0,0,0))
        model.setQuat(self.changeQuat * model.getQuat())
        self.velocityVector = model.getQuat().getForward() * speed
        model.setPos(  self.velocityVector[0] + model.getX(),
                            self.velocityVector[1] + model.getY(),
                            self.velocityVector[2] + model.getZ())


#***********************************************************
if __name__ == '__main__':
    w = World()
    run()
